@@ -23,95 +23,48 @@
 #include <linux/firmware.h>
 #include "drmP.h"
 #include "amdgpu.h"
-#include "gmc_v8_0.h"
+#include "cikd.h"
+#include "cik.h"
+#include "gmc_v7_0.h"
 #include "amdgpu_ucode.h"
 
-#include "gmc/gmc_8_1_d.h"
-#include "gmc/gmc_8_1_sh_mask.h"
+#include "bif/bif_4_1_d.h"
+#include "bif/bif_4_1_sh_mask.h"
 
-#include "bif/bif_5_0_d.h"
-#include "bif/bif_5_0_sh_mask.h"
+#include "gmc/gmc_7_1_d.h"
+#include "gmc/gmc_7_1_sh_mask.h"
 
-#include "oss/oss_3_0_d.h"
-#include "oss/oss_3_0_sh_mask.h"
+#include "oss/oss_2_0_d.h"
+#include "oss/oss_2_0_sh_mask.h"
 
-#include "vid.h"
-#include "vi.h"
-
-
-static void gmc_v8_0_set_gart_funcs(struct amdgpu_device *adev);
-static void gmc_v8_0_set_irq_funcs(struct amdgpu_device *adev);
+static void gmc_v7_0_set_gart_funcs(struct amdgpu_device *adev);
+static void gmc_v7_0_set_irq_funcs(struct amdgpu_device *adev);
 
 /*(DEBLOBBED)*/
 
-static const u32 golden_settings_tonga_a11[] =
-{
-	mmMC_ARB_WTM_GRPWT_RD, 0x00000003, 0x00000000,
-	mmMC_HUB_RDREQ_DMIF_LIMIT, 0x0000007f, 0x00000028,
-	mmMC_HUB_WDP_UMC, 0x00007fb6, 0x00000991,
-	mmVM_PRT_APERTURE0_LOW_ADDR, 0x0fffffff, 0x0fffffff,
-	mmVM_PRT_APERTURE1_LOW_ADDR, 0x0fffffff, 0x0fffffff,
-	mmVM_PRT_APERTURE2_LOW_ADDR, 0x0fffffff, 0x0fffffff,
-	mmVM_PRT_APERTURE3_LOW_ADDR, 0x0fffffff, 0x0fffffff,
-};
-
-static const u32 tonga_mgcg_cgcg_init[] =
-{
-	mmMC_MEM_POWER_LS, 0xffffffff, 0x00000104
-};
-
-static const u32 golden_settings_fiji_a10[] =
+static const u32 golden_settings_iceland_a11[] =
 {
 	mmVM_PRT_APERTURE0_LOW_ADDR, 0x0fffffff, 0x0fffffff,
 	mmVM_PRT_APERTURE1_LOW_ADDR, 0x0fffffff, 0x0fffffff,
 	mmVM_PRT_APERTURE2_LOW_ADDR, 0x0fffffff, 0x0fffffff,
-	mmVM_PRT_APERTURE3_LOW_ADDR, 0x0fffffff, 0x0fffffff,
+	mmVM_PRT_APERTURE3_LOW_ADDR, 0x0fffffff, 0x0fffffff
 };
 
-static const u32 fiji_mgcg_cgcg_init[] =
+static const u32 iceland_mgcg_cgcg_init[] =
 {
 	mmMC_MEM_POWER_LS, 0xffffffff, 0x00000104
 };
 
-static const u32 cz_mgcg_cgcg_init[] =
-{
-	mmMC_MEM_POWER_LS, 0xffffffff, 0x00000104
-};
-
-static const u32 stoney_mgcg_cgcg_init[] =
-{
-	mmMC_MEM_POWER_LS, 0xffffffff, 0x00000104
-};
-
-
-static void gmc_v8_0_init_golden_registers(struct amdgpu_device *adev)
+static void gmc_v7_0_init_golden_registers(struct amdgpu_device *adev)
 {
 	switch (adev->asic_type) {
-	case CHIP_FIJI:
+	case CHIP_TOPAZ:
 		amdgpu_program_register_sequence(adev,
-						 fiji_mgcg_cgcg_init,
-						 (const u32)ARRAY_SIZE(fiji_mgcg_cgcg_init));
+						 iceland_mgcg_cgcg_init,
+						 (const u32)ARRAY_SIZE(iceland_mgcg_cgcg_init));
 		amdgpu_program_register_sequence(adev,
-						 golden_settings_fiji_a10,
-						 (const u32)ARRAY_SIZE(golden_settings_fiji_a10));
-		break;
-	case CHIP_TONGA:
-		amdgpu_program_register_sequence(adev,
-						 tonga_mgcg_cgcg_init,
-						 (const u32)ARRAY_SIZE(tonga_mgcg_cgcg_init));
-		amdgpu_program_register_sequence(adev,
-						 golden_settings_tonga_a11,
-						 (const u32)ARRAY_SIZE(golden_settings_tonga_a11));
-		break;
-	case CHIP_CARRIZO:
-		amdgpu_program_register_sequence(adev,
-						 cz_mgcg_cgcg_init,
-						 (const u32)ARRAY_SIZE(cz_mgcg_cgcg_init));
-		break;
-	case CHIP_STONEY:
-		amdgpu_program_register_sequence(adev,
-						 stoney_mgcg_cgcg_init,
-						 (const u32)ARRAY_SIZE(stoney_mgcg_cgcg_init));
+						 golden_settings_iceland_a11,
+						 (const u32)ARRAY_SIZE(golden_settings_iceland_a11));
 		break;
 	default:
 		break;
@@ -119,7 +72,7 @@ static void gmc_v8_0_init_golden_registers(struct amdgpu_device *adev)
 }
 
 /**
- * gmc8_mc_wait_for_idle - wait for MC idle callback.
+ * gmc7_mc_wait_for_idle - wait for MC idle callback.
  *
  * @adev: amdgpu_device pointer
  *
@@ -127,19 +80,14 @@ static void gmc_v8_0_init_golden_registers(struct amdgpu_device *adev)
  * (evergreen+).
  * Returns 0 if the MC is idle, -1 if not.
  */
-int gmc_v8_0_mc_wait_for_idle(struct amdgpu_device *adev)
+int gmc_v7_0_mc_wait_for_idle(struct amdgpu_device *adev)
 {
 	unsigned i;
 	u32 tmp;
 
 	for (i = 0; i < adev->usec_timeout; i++) {
 		/* read MC_STATUS */
-		tmp = RREG32(mmSRBM_STATUS) & (SRBM_STATUS__VMC_BUSY_MASK |
-					       SRBM_STATUS__MCB_BUSY_MASK |
-					       SRBM_STATUS__MCB_NON_DISPLAY_BUSY_MASK |
-					       SRBM_STATUS__MCC_BUSY_MASK |
-					       SRBM_STATUS__MCD_BUSY_MASK |
-					       SRBM_STATUS__VMC1_BUSY_MASK);
+		tmp = RREG32(mmSRBM_STATUS) & 0x1F00;
 		if (!tmp)
 			return 0;
 		udelay(1);
@@ -147,7 +95,7 @@ int gmc_v8_0_mc_wait_for_idle(struct amdgpu_device *adev)
 	return -1;
 }
 
-void gmc_v8_0_mc_stop(struct amdgpu_device *adev,
+void gmc_v7_0_mc_stop(struct amdgpu_device *adev,
 		      struct amdgpu_mode_mc_save *save)
 {
 	u32 blackout;
@@ -163,14 +111,14 @@ void gmc_v8_0_mc_stop(struct amdgpu_device *adev,
 		WREG32(mmBIF_FB_EN, 0);
 		/* blackout the MC */
 		blackout = REG_SET_FIELD(blackout,
-					 MC_SHARED_BLACKOUT_CNTL, BLACKOUT_MODE, 1);
-		WREG32(mmMC_SHARED_BLACKOUT_CNTL, blackout);
+					 MC_SHARED_BLACKOUT_CNTL, BLACKOUT_MODE, 0);
+		WREG32(mmMC_SHARED_BLACKOUT_CNTL, blackout | 1);
 	}
 	/* wait for the MC to settle */
 	udelay(100);
 }
 
-void gmc_v8_0_mc_resume(struct amdgpu_device *adev,
+void gmc_v7_0_mc_resume(struct amdgpu_device *adev,
 			struct amdgpu_mode_mc_save *save)
 {
 	u32 tmp;
@@ -189,7 +137,7 @@ void gmc_v8_0_mc_resume(struct amdgpu_device *adev,
 }
 
 /**
- * gmc_v8_0_init_microcode - load ucode images from disk
+ * gmc_v7_0_init_microcode - load ucode images from disk
  *
  * @adev: amdgpu_device pointer
  *
@@ -197,7 +145,7 @@ void gmc_v8_0_mc_resume(struct amdgpu_device *adev,
  * the driver (not loaded into hw).
  * Returns 0 on success, error on failure.
  */
-static int gmc_v8_0_init_microcode(struct amdgpu_device *adev)
+static int gmc_v7_0_init_microcode(struct amdgpu_device *adev)
 {
 	const char *chip_name;
 	char fw_name[30];
@@ -206,17 +154,26 @@ static int gmc_v8_0_init_microcode(struct amdgpu_device *adev)
 	DRM_DEBUG("\n");
 
 	switch (adev->asic_type) {
-	case CHIP_TONGA:
-		chip_name = "tonga";
+	case CHIP_BONAIRE:
+		chip_name = "bonaire";
 		break;
-	case CHIP_FIJI:
-	case CHIP_CARRIZO:
-	case CHIP_STONEY:
+	case CHIP_HAWAII:
+		chip_name = "hawaii";
+		break;
+	case CHIP_TOPAZ:
+		chip_name = "topaz";
+		break;
+	case CHIP_KAVERI:
+	case CHIP_KABINI:
 		return 0;
 	default: BUG();
 	}
 
-	snprintf(fw_name, sizeof(fw_name), "/*(DEBLOBBED)*/", chip_name);
+	if (adev->asic_type == CHIP_TOPAZ)
+		snprintf(fw_name, sizeof(fw_name), "/*(DEBLOBBED)*/", chip_name);
+	else
+		snprintf(fw_name, sizeof(fw_name), "/*(DEBLOBBED)*/", chip_name);
+
 	err = reject_firmware(&adev->mc.fw, fw_name, adev->dev);
 	if (err)
 		goto out;
@@ -225,7 +182,7 @@ static int gmc_v8_0_init_microcode(struct amdgpu_device *adev)
 out:
 	if (err) {
 		printk(KERN_ERR
-		       "mc: Failed to load firmware \"%s\"\n",
+		       "cik_mc: Failed to load firmware \"%s\"\n",
 		       fw_name);
 		release_firmware(adev->mc.fw);
 		adev->mc.fw = NULL;
@@ -234,14 +191,14 @@ out:
 }
 
 /**
- * gmc_v8_0_mc_load_microcode - load MC ucode into the hw
+ * gmc_v7_0_mc_load_microcode - load MC ucode into the hw
  *
  * @adev: amdgpu_device pointer
  *
  * Load the GDDR MC ucode into the hw (CIK).
  * Returns 0 on success, error on failure.
  */
-static int gmc_v8_0_mc_load_microcode(struct amdgpu_device *adev)
+static int gmc_v7_0_mc_load_microcode(struct amdgpu_device *adev)
 {
 	const struct mc_firmware_header_v1_0 *hdr;
 	const __le32 *fw_data = NULL;
@@ -310,7 +267,7 @@ static int gmc_v8_0_mc_load_microcode(struct amdgpu_device *adev)
 	return 0;
 }
 
-static void gmc_v8_0_vram_gtt_location(struct amdgpu_device *adev,
+static void gmc_v7_0_vram_gtt_location(struct amdgpu_device *adev,
 				       struct amdgpu_mc *mc)
 {
 	if (mc->mc_vram_size > 0xFFC0000000ULL) {
@@ -325,14 +282,14 @@ static void gmc_v8_0_vram_gtt_location(struct amdgpu_device *adev,
 }
 
 /**
- * gmc_v8_0_mc_program - program the GPU memory controller
+ * gmc_v7_0_mc_program - program the GPU memory controller
  *
  * @adev: amdgpu_device pointer
  *
  * Set the location of vram, gart, and AGP in the GPU's
  * physical address space (CIK).
  */
-static void gmc_v8_0_mc_program(struct amdgpu_device *adev)
+static void gmc_v7_0_mc_program(struct amdgpu_device *adev)
 {
 	struct amdgpu_mode_mc_save save;
 	u32 tmp;
@@ -351,7 +308,7 @@ static void gmc_v8_0_mc_program(struct amdgpu_device *adev)
 	if (adev->mode_info.num_crtc)
 		amdgpu_display_set_vga_render_state(adev, false);
 
-	gmc_v8_0_mc_stop(adev, &save);
+	gmc_v7_0_mc_stop(adev, &save);
 	if (amdgpu_asic_wait_for_mc_idle(adev)) {
 		dev_warn(adev->dev, "Wait for MC idle timedout !\n");
 	}
@@ -375,7 +332,7 @@ static void gmc_v8_0_mc_program(struct amdgpu_device *adev)
 	if (amdgpu_asic_wait_for_mc_idle(adev)) {
 		dev_warn(adev->dev, "Wait for MC idle timedout !\n");
 	}
-	gmc_v8_0_mc_resume(adev, &save);
+	gmc_v7_0_mc_resume(adev, &save);
 
 	WREG32(mmBIF_FB_EN, BIF_FB_EN__FB_READ_EN_MASK | BIF_FB_EN__FB_WRITE_EN_MASK);
 
@@ -388,7 +345,7 @@ static void gmc_v8_0_mc_program(struct amdgpu_device *adev)
 }
 
 /**
- * gmc_v8_0_mc_init - initialize the memory controller driver params
+ * gmc_v7_0_mc_init - initialize the memory controller driver params
  *
  * @adev: amdgpu_device pointer
  *
@@ -396,7 +353,7 @@ static void gmc_v8_0_mc_program(struct amdgpu_device *adev)
  * vram and gart within the GPU's physical address space (CIK).
  * Returns 0 for success.
  */
-static int gmc_v8_0_mc_init(struct amdgpu_device *adev)
+static int gmc_v7_0_mc_init(struct amdgpu_device *adev)
 {
 	u32 tmp;
 	int chansize, numchan;
@@ -456,7 +413,7 @@ static int gmc_v8_0_mc_init(struct amdgpu_device *adev)
 	else
 		adev->mc.gtt_size = (uint64_t)amdgpu_gart_size << 20;
 
-	gmc_v8_0_vram_gtt_location(adev, &adev->mc);
+	gmc_v7_0_vram_gtt_location(adev, &adev->mc);
 
 	return 0;
 }
@@ -469,14 +426,14 @@ static int gmc_v8_0_mc_init(struct amdgpu_device *adev)
  */
 
 /**
- * gmc_v8_0_gart_flush_gpu_tlb - gart tlb flush callback
+ * gmc_v7_0_gart_flush_gpu_tlb - gart tlb flush callback
  *
  * @adev: amdgpu_device pointer
  * @vmid: vm instance to flush
  *
  * Flush the TLB for the requested page table (CIK).
  */
-static void gmc_v8_0_gart_flush_gpu_tlb(struct amdgpu_device *adev,
+static void gmc_v7_0_gart_flush_gpu_tlb(struct amdgpu_device *adev,
 					uint32_t vmid)
 {
 	/* flush hdp cache */
@@ -487,7 +444,7 @@ static void gmc_v8_0_gart_flush_gpu_tlb(struct amdgpu_device *adev,
 }
 
 /**
- * gmc_v8_0_gart_set_pte_pde - update the page tables using MMIO
+ * gmc_v7_0_gart_set_pte_pde - update the page tables using MMIO
  *
  * @adev: amdgpu_device pointer
  * @cpu_pt_addr: cpu address of the page table
@@ -497,7 +454,7 @@ static void gmc_v8_0_gart_flush_gpu_tlb(struct amdgpu_device *adev,
  *
  * Update the page tables using the CPU.
  */
-static int gmc_v8_0_gart_set_pte_pde(struct amdgpu_device *adev,
+static int gmc_v7_0_gart_set_pte_pde(struct amdgpu_device *adev,
 				     void *cpu_pt_addr,
 				     uint32_t gpu_page_idx,
 				     uint64_t addr,
@@ -506,27 +463,7 @@ static int gmc_v8_0_gart_set_pte_pde(struct amdgpu_device *adev,
 	void __iomem *ptr = (void *)cpu_pt_addr;
 	uint64_t value;
 
-	/*
-	 * PTE format on VI:
-	 * 63:40 reserved
-	 * 39:12 4k physical page base address
-	 * 11:7 fragment
-	 * 6 write
-	 * 5 read
-	 * 4 exe
-	 * 3 reserved
-	 * 2 snooped
-	 * 1 system
-	 * 0 valid
-	 *
-	 * PDE format on VI:
-	 * 63:59 block fragment size
-	 * 58:40 reserved
-	 * 39:1 physical base address of PTE
-	 * bits 5:1 must be 0.
-	 * 0 valid
-	 */
-	value = addr & 0x000000FFFFFFF000ULL;
+	value = addr & 0xFFFFFFFFFFFFF000ULL;
 	value |= flags;
 	writeq(value, ptr + (gpu_page_idx * 8));
 
@@ -539,7 +476,7 @@ static int gmc_v8_0_gart_set_pte_pde(struct amdgpu_device *adev,
  * @adev: amdgpu_device pointer
  * @value: true redirects VM faults to the default page
  */
-static void gmc_v8_0_set_fault_enable_default(struct amdgpu_device *adev,
+static void gmc_v7_0_set_fault_enable_default(struct amdgpu_device *adev,
 					      bool value)
 {
 	u32 tmp;
@@ -557,13 +494,11 @@ static void gmc_v8_0_set_fault_enable_default(struct amdgpu_device *adev,
 			    READ_PROTECTION_FAULT_ENABLE_DEFAULT, value);
 	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL,
 			    WRITE_PROTECTION_FAULT_ENABLE_DEFAULT, value);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL,
-			    EXECUTE_PROTECTION_FAULT_ENABLE_DEFAULT, value);
 	WREG32(mmVM_CONTEXT1_CNTL, tmp);
 }
 
 /**
- * gmc_v8_0_gart_enable - gart enable
+ * gmc_v7_0_gart_enable - gart enable
  *
  * @adev: amdgpu_device pointer
  *
@@ -573,7 +508,7 @@ static void gmc_v8_0_set_fault_enable_default(struct amdgpu_device *adev,
  * and GPUVM for FSA64 clients (CIK).
  * Returns 0 for success, errors for failure.
  */
-static int gmc_v8_0_gart_enable(struct amdgpu_device *adev)
+static int gmc_v7_0_gart_enable(struct amdgpu_device *adev)
 {
 	int r, i;
 	u32 tmp;
@@ -603,8 +538,7 @@ static int gmc_v8_0_gart_enable(struct amdgpu_device *adev)
 	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL, CONTEXT1_IDENTITY_ACCESS_MODE, 1);
 	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL, ENABLE_DEFAULT_PAGE_OUT_TO_SYSTEM_MEMORY, 1);
 	WREG32(mmVM_L2_CNTL, tmp);
-	tmp = RREG32(mmVM_L2_CNTL2);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL2, INVALIDATE_ALL_L1_TLBS, 1);
+	tmp = REG_SET_FIELD(0, VM_L2_CNTL2, INVALIDATE_ALL_L1_TLBS, 1);
 	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL2, INVALIDATE_L2_CACHE, 1);
 	WREG32(mmVM_L2_CNTL2, tmp);
 	tmp = RREG32(mmVM_L2_CNTL3);
@@ -612,21 +546,6 @@ static int gmc_v8_0_gart_enable(struct amdgpu_device *adev)
 	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL3, BANK_SELECT, 4);
 	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL3, L2_CACHE_BIGK_FRAGMENT_SIZE, 4);
 	WREG32(mmVM_L2_CNTL3, tmp);
-	/* XXX: set to enable PTE/PDE in system memory */
-	tmp = RREG32(mmVM_L2_CNTL4);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT0_PDE_REQUEST_PHYSICAL, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT0_PDE_REQUEST_SHARED, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT0_PDE_REQUEST_SNOOP, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT0_PTE_REQUEST_PHYSICAL, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT0_PTE_REQUEST_SHARED, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT0_PTE_REQUEST_SNOOP, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT1_PDE_REQUEST_PHYSICAL, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT1_PDE_REQUEST_SHARED, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT1_PDE_REQUEST_SNOOP, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT1_PTE_REQUEST_PHYSICAL, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT1_PTE_REQUEST_SHARED, 0);
-	tmp = REG_SET_FIELD(tmp, VM_L2_CNTL4, VMC_TAP_CONTEXT1_PTE_REQUEST_SNOOP, 0);
-	WREG32(mmVM_L2_CNTL4, tmp);
 	/* setup context0 */
 	WREG32(mmVM_CONTEXT0_PAGE_TABLE_START_ADDR, adev->mc.gtt_start >> 12);
 	WREG32(mmVM_CONTEXT0_PAGE_TABLE_END_ADDR, adev->mc.gtt_end >> 12);
@@ -640,9 +559,9 @@ static int gmc_v8_0_gart_enable(struct amdgpu_device *adev)
 	tmp = REG_SET_FIELD(tmp, VM_CONTEXT0_CNTL, RANGE_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
 	WREG32(mmVM_CONTEXT0_CNTL, tmp);
 
-	WREG32(mmVM_L2_CONTEXT1_IDENTITY_APERTURE_LOW_ADDR, 0);
-	WREG32(mmVM_L2_CONTEXT1_IDENTITY_APERTURE_HIGH_ADDR, 0);
-	WREG32(mmVM_L2_CONTEXT_IDENTITY_PHYSICAL_OFFSET, 0);
+	WREG32(0x575, 0);
+	WREG32(0x576, 0);
+	WREG32(0x577, 0);
 
 	/* empty context1-15 */
 	/* FIXME start with 4G, once using 2 level pt switch to full
@@ -667,22 +586,21 @@ static int gmc_v8_0_gart_enable(struct amdgpu_device *adev)
 	tmp = RREG32(mmVM_CONTEXT1_CNTL);
 	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, ENABLE_CONTEXT, 1);
 	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, PAGE_TABLE_DEPTH, 1);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, RANGE_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, DUMMY_PAGE_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, PDE0_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, VALID_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, READ_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, WRITE_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
-	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, EXECUTE_PROTECTION_FAULT_ENABLE_DEFAULT, 1);
 	tmp = REG_SET_FIELD(tmp, VM_CONTEXT1_CNTL, PAGE_TABLE_BLOCK_SIZE,
 			    amdgpu_vm_block_size - 9);
 	WREG32(mmVM_CONTEXT1_CNTL, tmp);
 	if (amdgpu_vm_fault_stop == AMDGPU_VM_FAULT_STOP_ALWAYS)
-		gmc_v8_0_set_fault_enable_default(adev, false);
+		gmc_v7_0_set_fault_enable_default(adev, false);
 	else
-		gmc_v8_0_set_fault_enable_default(adev, true);
+		gmc_v7_0_set_fault_enable_default(adev, true);
 
-	gmc_v8_0_gart_flush_gpu_tlb(adev, 0);
+	if (adev->asic_type == CHIP_KAVERI) {
+		tmp = RREG32(mmCHUB_CONTROL);
+		tmp &= ~BYPASS_VM;
+		WREG32(mmCHUB_CONTROL, tmp);
+	}
+
+	gmc_v7_0_gart_flush_gpu_tlb(adev, 0);
 	DRM_INFO("PCIE GART of %uM enabled (table at 0x%016llX).\n",
 		 (unsigned)(adev->mc.gtt_size >> 20),
 		 (unsigned long long)adev->gart.table_addr);
@@ -690,7 +608,7 @@ static int gmc_v8_0_gart_enable(struct amdgpu_device *adev)
 	return 0;
 }
 
-static int gmc_v8_0_gart_init(struct amdgpu_device *adev)
+static int gmc_v7_0_gart_init(struct amdgpu_device *adev)
 {
 	int r;
 
@@ -707,13 +625,13 @@ static int gmc_v8_0_gart_init(struct amdgpu_device *adev)
 }
 
 /**
- * gmc_v8_0_gart_disable - gart disable
+ * gmc_v7_0_gart_disable - gart disable
  *
  * @adev: amdgpu_device pointer
  *
  * This disables all VM page table (CIK).
  */
-static void gmc_v8_0_gart_disable(struct amdgpu_device *adev)
+static void gmc_v7_0_gart_disable(struct amdgpu_device *adev)
 {
 	u32 tmp;
 
@@ -735,13 +653,13 @@ static void gmc_v8_0_gart_disable(struct amdgpu_device *adev)
 }
 
 /**
- * gmc_v8_0_gart_fini - vm fini callback
+ * gmc_v7_0_gart_fini - vm fini callback
  *
  * @adev: amdgpu_device pointer
  *
  * Tears down the driver GART/VM setup (CIK).
  */
-static void gmc_v8_0_gart_fini(struct amdgpu_device *adev)
+static void gmc_v7_0_gart_fini(struct amdgpu_device *adev)
 {
 	amdgpu_gart_table_vram_free(adev);
 	amdgpu_gart_fini(adev);
@@ -754,7 +672,7 @@ static void gmc_v8_0_gart_fini(struct amdgpu_device *adev)
  * by the amdgpu vm/hsa code.
  */
 /**
- * gmc_v8_0_vm_init - cik vm init callback
+ * gmc_v7_0_vm_init - cik vm init callback
  *
  * @adev: amdgpu_device pointer
  *
@@ -762,7 +680,7 @@ static void gmc_v8_0_gart_fini(struct amdgpu_device *adev)
  * VMIDs 1-15) (CIK).
  * Returns 0 for success.
  */
-static int gmc_v8_0_vm_init(struct amdgpu_device *adev)
+static int gmc_v7_0_vm_init(struct amdgpu_device *adev)
 {
 	/*
 	 * number of VMs
@@ -784,18 +702,18 @@ static int gmc_v8_0_vm_init(struct amdgpu_device *adev)
 }
 
 /**
- * gmc_v8_0_vm_fini - cik vm fini callback
+ * gmc_v7_0_vm_fini - cik vm fini callback
  *
  * @adev: amdgpu_device pointer
  *
  * Tear down any asic specific VM setup (CIK).
  */
-static void gmc_v8_0_vm_fini(struct amdgpu_device *adev)
+static void gmc_v7_0_vm_fini(struct amdgpu_device *adev)
 {
 }
 
 /**
- * gmc_v8_0_vm_decode_fault - print human readable fault info
+ * gmc_v7_0_vm_decode_fault - print human readable fault info
  *
  * @adev: amdgpu_device pointer
  * @status: VM_CONTEXT1_PROTECTION_FAULT_STATUS register value
@@ -803,7 +721,7 @@ static void gmc_v8_0_vm_fini(struct amdgpu_device *adev)
  *
  * Print human readable fault information (CIK).
  */
-static void gmc_v8_0_vm_decode_fault(struct amdgpu_device *adev,
+static void gmc_v7_0_vm_decode_fault(struct amdgpu_device *adev,
 				     u32 status, u32 addr, u32 mc_client)
 {
 	u32 mc_id;
@@ -823,7 +741,133 @@ static void gmc_v8_0_vm_decode_fault(struct amdgpu_device *adev,
 	       "write" : "read", block, mc_client, mc_id);
 }
 
-static int gmc_v8_0_convert_vram_type(int mc_seq_vram_type)
+
+static const u32 mc_cg_registers[] = {
+	mmMC_HUB_MISC_HUB_CG,
+	mmMC_HUB_MISC_SIP_CG,
+	mmMC_HUB_MISC_VM_CG,
+	mmMC_XPB_CLK_GAT,
+	mmATC_MISC_CG,
+	mmMC_CITF_MISC_WR_CG,
+	mmMC_CITF_MISC_RD_CG,
+	mmMC_CITF_MISC_VM_CG,
+	mmVM_L2_CG,
+};
+
+static const u32 mc_cg_ls_en[] = {
+	MC_HUB_MISC_HUB_CG__MEM_LS_ENABLE_MASK,
+	MC_HUB_MISC_SIP_CG__MEM_LS_ENABLE_MASK,
+	MC_HUB_MISC_VM_CG__MEM_LS_ENABLE_MASK,
+	MC_XPB_CLK_GAT__MEM_LS_ENABLE_MASK,
+	ATC_MISC_CG__MEM_LS_ENABLE_MASK,
+	MC_CITF_MISC_WR_CG__MEM_LS_ENABLE_MASK,
+	MC_CITF_MISC_RD_CG__MEM_LS_ENABLE_MASK,
+	MC_CITF_MISC_VM_CG__MEM_LS_ENABLE_MASK,
+	VM_L2_CG__MEM_LS_ENABLE_MASK,
+};
+
+static const u32 mc_cg_en[] = {
+	MC_HUB_MISC_HUB_CG__ENABLE_MASK,
+	MC_HUB_MISC_SIP_CG__ENABLE_MASK,
+	MC_HUB_MISC_VM_CG__ENABLE_MASK,
+	MC_XPB_CLK_GAT__ENABLE_MASK,
+	ATC_MISC_CG__ENABLE_MASK,
+	MC_CITF_MISC_WR_CG__ENABLE_MASK,
+	MC_CITF_MISC_RD_CG__ENABLE_MASK,
+	MC_CITF_MISC_VM_CG__ENABLE_MASK,
+	VM_L2_CG__ENABLE_MASK,
+};
+
+static void gmc_v7_0_enable_mc_ls(struct amdgpu_device *adev,
+				  bool enable)
+{
+	int i;
+	u32 orig, data;
+
+	for (i = 0; i < ARRAY_SIZE(mc_cg_registers); i++) {
+		orig = data = RREG32(mc_cg_registers[i]);
+		if (enable && (adev->cg_flags & AMDGPU_CG_SUPPORT_MC_LS))
+			data |= mc_cg_ls_en[i];
+		else
+			data &= ~mc_cg_ls_en[i];
+		if (data != orig)
+			WREG32(mc_cg_registers[i], data);
+	}
+}
+
+static void gmc_v7_0_enable_mc_mgcg(struct amdgpu_device *adev,
+				    bool enable)
+{
+	int i;
+	u32 orig, data;
+
+	for (i = 0; i < ARRAY_SIZE(mc_cg_registers); i++) {
+		orig = data = RREG32(mc_cg_registers[i]);
+		if (enable && (adev->cg_flags & AMDGPU_CG_SUPPORT_MC_MGCG))
+			data |= mc_cg_en[i];
+		else
+			data &= ~mc_cg_en[i];
+		if (data != orig)
+			WREG32(mc_cg_registers[i], data);
+	}
+}
+
+static void gmc_v7_0_enable_bif_mgls(struct amdgpu_device *adev,
+				     bool enable)
+{
+	u32 orig, data;
+
+	orig = data = RREG32_PCIE(ixPCIE_CNTL2);
+
+	if (enable && (adev->cg_flags & AMDGPU_CG_SUPPORT_BIF_LS)) {
+		data = REG_SET_FIELD(data, PCIE_CNTL2, SLV_MEM_LS_EN, 1);
+		data = REG_SET_FIELD(data, PCIE_CNTL2, MST_MEM_LS_EN, 1);
+		data = REG_SET_FIELD(data, PCIE_CNTL2, REPLAY_MEM_LS_EN, 1);
+		data = REG_SET_FIELD(data, PCIE_CNTL2, SLV_MEM_AGGRESSIVE_LS_EN, 1);
+	} else {
+		data = REG_SET_FIELD(data, PCIE_CNTL2, SLV_MEM_LS_EN, 0);
+		data = REG_SET_FIELD(data, PCIE_CNTL2, MST_MEM_LS_EN, 0);
+		data = REG_SET_FIELD(data, PCIE_CNTL2, REPLAY_MEM_LS_EN, 0);
+		data = REG_SET_FIELD(data, PCIE_CNTL2, SLV_MEM_AGGRESSIVE_LS_EN, 0);
+	}
+
+	if (orig != data)
+		WREG32_PCIE(ixPCIE_CNTL2, data);
+}
+
+static void gmc_v7_0_enable_hdp_mgcg(struct amdgpu_device *adev,
+				     bool enable)
+{
+	u32 orig, data;
+
+	orig = data = RREG32(mmHDP_HOST_PATH_CNTL);
+
+	if (enable && (adev->cg_flags & AMDGPU_CG_SUPPORT_HDP_MGCG))
+		data = REG_SET_FIELD(data, HDP_HOST_PATH_CNTL, CLOCK_GATING_DIS, 0);
+	else
+		data = REG_SET_FIELD(data, HDP_HOST_PATH_CNTL, CLOCK_GATING_DIS, 1);
+
+	if (orig != data)
+		WREG32(mmHDP_HOST_PATH_CNTL, data);
+}
+
+static void gmc_v7_0_enable_hdp_ls(struct amdgpu_device *adev,
+				   bool enable)
+{
+	u32 orig, data;
+
+	orig = data = RREG32(mmHDP_MEM_POWER_LS);
+
+	if (enable && (adev->cg_flags & AMDGPU_CG_SUPPORT_HDP_LS))
+		data = REG_SET_FIELD(data, HDP_MEM_POWER_LS, LS_ENABLE, 1);
+	else
+		data = REG_SET_FIELD(data, HDP_MEM_POWER_LS, LS_ENABLE, 0);
+
+	if (orig != data)
+		WREG32(mmHDP_MEM_POWER_LS, data);
+}
+
+static int gmc_v7_0_convert_vram_type(int mc_seq_vram_type)
 {
 	switch (mc_seq_vram_type) {
 	case MC_SEQ_MISC0__MT__GDDR1:
@@ -845,32 +889,24 @@ static int gmc_v8_0_convert_vram_type(int mc_seq_vram_type)
 	}
 }
 
-static int gmc_v8_0_early_init(void *handle)
+static int gmc_v7_0_early_init(void *handle)
 {
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 
-	gmc_v8_0_set_gart_funcs(adev);
-	gmc_v8_0_set_irq_funcs(adev);
-
-	if (adev->flags & AMD_IS_APU) {
-		adev->mc.vram_type = AMDGPU_VRAM_TYPE_UNKNOWN;
-	} else {
-		u32 tmp = RREG32(mmMC_SEQ_MISC0);
-		tmp &= MC_SEQ_MISC0__MT__MASK;
-		adev->mc.vram_type = gmc_v8_0_convert_vram_type(tmp);
-	}
+	gmc_v7_0_set_gart_funcs(adev);
+	gmc_v7_0_set_irq_funcs(adev);
 
 	return 0;
 }
 
-static int gmc_v8_0_late_init(void *handle)
+static int gmc_v7_0_late_init(void *handle)
 {
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 
 	return amdgpu_irq_get(adev, &adev->mc.vm_fault, 0);
 }
 
-static int gmc_v8_0_sw_init(void *handle)
+static int gmc_v7_0_sw_init(void *handle)
 {
 	int r;
 	int dma_bits;
@@ -880,6 +916,14 @@ static int gmc_v8_0_sw_init(void *handle)
 	if (r)
 		return r;
 
+	if (adev->flags & AMD_IS_APU) {
+		adev->mc.vram_type = AMDGPU_VRAM_TYPE_UNKNOWN;
+	} else {
+		u32 tmp = RREG32(mmMC_SEQ_MISC0);
+		tmp &= MC_SEQ_MISC0__MT__MASK;
+		adev->mc.vram_type = gmc_v7_0_convert_vram_type(tmp);
+	}
+
 	r = amdgpu_irq_add_id(adev, 146, &adev->mc.vm_fault);
 	if (r)
 		return r;
@@ -919,13 +963,13 @@ static int gmc_v8_0_sw_init(void *handle)
 		printk(KERN_WARNING "amdgpu: No coherent DMA available.\n");
 	}
 
-	r = gmc_v8_0_init_microcode(adev);
+	r = gmc_v7_0_init_microcode(adev);
 	if (r) {
 		DRM_ERROR("Failed to load mc firmware!\n");
 		return r;
 	}
 
-	r = gmc_v8_0_mc_init(adev);
+	r = gmc_v7_0_mc_init(adev);
 	if (r)
 		return r;
 
@@ -934,12 +978,12 @@ static int gmc_v8_0_sw_init(void *handle)
 	if (r)
 		return r;
 
-	r = gmc_v8_0_gart_init(adev);
+	r = gmc_v7_0_gart_init(adev);
 	if (r)
 		return r;
 
 	if (!adev->vm_manager.enabled) {
-		r = gmc_v8_0_vm_init(adev);
+		r = gmc_v7_0_vm_init(adev);
 		if (r) {
 			dev_err(adev->dev, "vm manager initialization failed (%d).\n", r);
 			return r;
@@ -950,81 +994,81 @@ static int gmc_v8_0_sw_init(void *handle)
 	return r;
 }
 
-static int gmc_v8_0_sw_fini(void *handle)
+static int gmc_v7_0_sw_fini(void *handle)
 {
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 
 	if (adev->vm_manager.enabled) {
 		amdgpu_vm_manager_fini(adev);
-		gmc_v8_0_vm_fini(adev);
+		gmc_v7_0_vm_fini(adev);
 		adev->vm_manager.enabled = false;
 	}
-	gmc_v8_0_gart_fini(adev);
+	gmc_v7_0_gart_fini(adev);
 	amdgpu_gem_fini(adev);
 	amdgpu_bo_fini(adev);
 
 	return 0;
 }
 
-static int gmc_v8_0_hw_init(void *handle)
+static int gmc_v7_0_hw_init(void *handle)
 {
 	int r;
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 
-	gmc_v8_0_init_golden_registers(adev);
+	gmc_v7_0_init_golden_registers(adev);
 
-	gmc_v8_0_mc_program(adev);
+	gmc_v7_0_mc_program(adev);
 
-	if (adev->asic_type == CHIP_TONGA) {
-		r = gmc_v8_0_mc_load_microcode(adev);
+	if (!(adev->flags & AMD_IS_APU)) {
+		r = gmc_v7_0_mc_load_microcode(adev);
 		if (r) {
 			DRM_ERROR("Failed to load MC firmware!\n");
 			return r;
 		}
 	}
 
-	r = gmc_v8_0_gart_enable(adev);
+	r = gmc_v7_0_gart_enable(adev);
 	if (r)
 		return r;
 
 	return r;
 }
 
-static int gmc_v8_0_hw_fini(void *handle)
+static int gmc_v7_0_hw_fini(void *handle)
 {
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 
 	amdgpu_irq_put(adev, &adev->mc.vm_fault, 0);
-	gmc_v8_0_gart_disable(adev);
+	gmc_v7_0_gart_disable(adev);
 
 	return 0;
 }
 
-static int gmc_v8_0_suspend(void *handle)
+static int gmc_v7_0_suspend(void *handle)
 {
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 
 	if (adev->vm_manager.enabled) {
 		amdgpu_vm_manager_fini(adev);
-		gmc_v8_0_vm_fini(adev);
+		gmc_v7_0_vm_fini(adev);
 		adev->vm_manager.enabled = false;
 	}
-	gmc_v8_0_hw_fini(adev);
+	gmc_v7_0_hw_fini(adev);
 
 	return 0;
 }
 
-static int gmc_v8_0_resume(void *handle)
+static int gmc_v7_0_resume(void *handle)
 {
 	int r;
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 
-	r = gmc_v8_0_hw_init(adev);
+	r = gmc_v7_0_hw_init(adev);
 	if (r)
 		return r;
 
 	if (!adev->vm_manager.enabled) {
-		r = gmc_v8_0_vm_init(adev);
+		r = gmc_v7_0_vm_init(adev);
 		if (r) {
 			dev_err(adev->dev, "vm manager initialization failed (%d).\n", r);
 			return r;
@@ -1035,7 +1079,7 @@ static int gmc_v8_0_resume(void *handle)
 	return r;
 }
 
-static bool gmc_v8_0_is_idle(void *handle)
+static bool gmc_v7_0_is_idle(void *handle)
 {
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 	u32 tmp = RREG32(mmSRBM_STATUS);
@@ -1047,7 +1091,7 @@ static bool gmc_v8_0_is_idle(void *handle)
 	return true;
 }
 
-static int gmc_v8_0_wait_for_idle(void *handle)
+static int gmc_v7_0_wait_for_idle(void *handle)
 {
 	unsigned i;
 	u32 tmp;
@@ -1059,8 +1103,7 @@ static int gmc_v8_0_wait_for_idle(void *handle)
 					       SRBM_STATUS__MCB_NON_DISPLAY_BUSY_MASK |
 					       SRBM_STATUS__MCC_BUSY_MASK |
 					       SRBM_STATUS__MCD_BUSY_MASK |
-					       SRBM_STATUS__VMC_BUSY_MASK |
-					       SRBM_STATUS__VMC1_BUSY_MASK);
+					       SRBM_STATUS__VMC_BUSY_MASK);
 		if (!tmp)
 			return 0;
 		udelay(1);
@@ -1069,7 +1112,7 @@ static int gmc_v8_0_wait_for_idle(void *handle)
 
 }
 
-static void gmc_v8_0_print_status(void *handle)
+static void gmc_v7_0_print_status(void *handle)
 {
 	int i, j;
 	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
@@ -1092,8 +1135,6 @@ static void gmc_v8_0_print_status(void *handle)
 		 RREG32(mmVM_L2_CNTL2));
 	dev_info(adev->dev, "  VM_L2_CNTL3=0x%08X\n",
 		 RREG32(mmVM_L2_CNTL3));
-	dev_info(adev->dev, "  VM_L2_CNTL4=0x%08X\n",
-		 RREG32(mmVM_L2_CNTL4));
 	dev_info(adev->dev, "  VM_CONTEXT0_PAGE_TABLE_START_ADDR=0x%08X\n",
 		 RREG32(mmVM_CONTEXT0_PAGE_TABLE_START_ADDR));
 	dev_info(adev->dev, "  VM_CONTEXT0_PAGE_TABLE_END_ADDR=0x%08X\n",
@@ -1104,12 +1145,12 @@ static void gmc_v8_0_print_status(void *handle)
 		 RREG32(mmVM_CONTEXT0_CNTL2));
 	dev_info(adev->dev, "  VM_CONTEXT0_CNTL=0x%08X\n",
 		 RREG32(mmVM_CONTEXT0_CNTL));
-	dev_info(adev->dev, "  VM_L2_CONTEXT1_IDENTITY_APERTURE_LOW_ADDR=0x%08X\n",
-		 RREG32(mmVM_L2_CONTEXT1_IDENTITY_APERTURE_LOW_ADDR));
-	dev_info(adev->dev, "  VM_L2_CONTEXT1_IDENTITY_APERTURE_HIGH_ADDR=0x%08X\n",
-		 RREG32(mmVM_L2_CONTEXT1_IDENTITY_APERTURE_HIGH_ADDR));
-	dev_info(adev->dev, "  mmVM_L2_CONTEXT_IDENTITY_PHYSICAL_OFFSET=0x%08X\n",
-		 RREG32(mmVM_L2_CONTEXT_IDENTITY_PHYSICAL_OFFSET));
+	dev_info(adev->dev, "  0x15D4=0x%08X\n",
+		 RREG32(0x575));
+	dev_info(adev->dev, "  0x15D8=0x%08X\n",
+		 RREG32(0x576));
+	dev_info(adev->dev, "  0x15DC=0x%08X\n",
+		 RREG32(0x577));
 	dev_info(adev->dev, "  VM_CONTEXT1_PAGE_TABLE_START_ADDR=0x%08X\n",
 		 RREG32(mmVM_CONTEXT1_PAGE_TABLE_START_ADDR));
 	dev_info(adev->dev, "  VM_CONTEXT1_PAGE_TABLE_END_ADDR=0x%08X\n",
@@ -1143,6 +1184,11 @@ static void gmc_v8_0_print_status(void *handle)
 	dev_info(adev->dev, "  MC_VM_AGP_BOT=0x%08X\n",
 		 RREG32(mmMC_VM_AGP_BOT));
 
+	if (adev->asic_type == CHIP_KAVERI) {
+		dev_info(adev->dev, "  CHUB_CONTROL=0x%08X\n",
+			 RREG32(mmCHUB_CONTROL));
+	}
+
 	dev_info(adev->dev, "  HDP_REG_COHERENCY_FLUSH_CNTL=0x%08X\n",
 		 RREG32(mmHDP_REG_COHERENCY_FLUSH_CNTL));
 	dev_info(adev->dev, "  HDP_NONSURFACE_BASE=0x%08X\n",
@@ -1174,11 +1220,11 @@ static void gmc_v8_0_print_status(void *handle)
 		 RREG32(mmBIF_FB_EN));
 }
 
-static int gmc_v8_0_soft_reset(void *handle)
+static int gmc_v7_0_soft_reset(void *handle)
 {
+	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 	struct amdgpu_mode_mc_save save;
 	u32 srbm_soft_reset = 0;
-	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
 	u32 tmp = RREG32(mmSRBM_STATUS);
 
 	if (tmp & SRBM_STATUS__VMC_BUSY_MASK)
@@ -1193,10 +1239,10 @@ static int gmc_v8_0_soft_reset(void *handle)
 	}
 
 	if (srbm_soft_reset) {
-		gmc_v8_0_print_status((void *)adev);
+		gmc_v7_0_print_status((void *)adev);
 
-		gmc_v8_0_mc_stop(adev, &save);
-		if (gmc_v8_0_wait_for_idle(adev)) {
+		gmc_v7_0_mc_stop(adev, &save);
+		if (gmc_v7_0_wait_for_idle(adev)) {
 			dev_warn(adev->dev, "Wait for GMC idle timed out !\n");
 		}
 
@@ -1216,16 +1262,16 @@ static int gmc_v8_0_soft_reset(void *handle)
 		/* Wait a little for things to settle down */
 		udelay(50);
 
-		gmc_v8_0_mc_resume(adev, &save);
+		gmc_v7_0_mc_resume(adev, &save);
 		udelay(50);
 
-		gmc_v8_0_print_status((void *)adev);
+		gmc_v7_0_print_status((void *)adev);
 	}
 
 	return 0;
 }
 
-static int gmc_v8_0_vm_fault_interrupt_state(struct amdgpu_device *adev,
+static int gmc_v7_0_vm_fault_interrupt_state(struct amdgpu_device *adev,
 					     struct amdgpu_irq_src *src,
 					     unsigned type,
 					     enum amdgpu_interrupt_state state)
@@ -1236,8 +1282,7 @@ static int gmc_v8_0_vm_fault_interrupt_state(struct amdgpu_device *adev,
 		    VM_CONTEXT1_CNTL__PDE0_PROTECTION_FAULT_ENABLE_INTERRUPT_MASK |
 		    VM_CONTEXT1_CNTL__VALID_PROTECTION_FAULT_ENABLE_INTERRUPT_MASK |
 		    VM_CONTEXT1_CNTL__READ_PROTECTION_FAULT_ENABLE_INTERRUPT_MASK |
-		    VM_CONTEXT1_CNTL__WRITE_PROTECTION_FAULT_ENABLE_INTERRUPT_MASK |
-		    VM_CONTEXT1_CNTL__EXECUTE_PROTECTION_FAULT_ENABLE_INTERRUPT_MASK);
+		    VM_CONTEXT1_CNTL__WRITE_PROTECTION_FAULT_ENABLE_INTERRUPT_MASK);
 
 	switch (state) {
 	case AMDGPU_IRQ_STATE_DISABLE:
@@ -1267,7 +1312,7 @@ static int gmc_v8_0_vm_fault_interrupt_state(struct amdgpu_device *adev,
 	return 0;
 }
 
-static int gmc_v8_0_process_interrupt(struct amdgpu_device *adev,
+static int gmc_v7_0_process_interrupt(struct amdgpu_device *adev,
 				      struct amdgpu_irq_src *source,
 				      struct amdgpu_iv_entry *entry)
 {
@@ -1283,7 +1328,7 @@ static int gmc_v8_0_process_interrupt(struct amdgpu_device *adev,
 		return 0;
 
 	if (amdgpu_vm_fault_stop == AMDGPU_VM_FAULT_STOP_FIRST)
-		gmc_v8_0_set_fault_enable_default(adev, false);
+		gmc_v7_0_set_fault_enable_default(adev, false);
 
 	dev_err(adev->dev, "GPU fault detected: %d 0x%08x\n",
 		entry->src_id, entry->src_data);
@@ -1291,58 +1336,72 @@ static int gmc_v8_0_process_interrupt(struct amdgpu_device *adev,
 		addr);
 	dev_err(adev->dev, "  VM_CONTEXT1_PROTECTION_FAULT_STATUS 0x%08X\n",
 		status);
-	gmc_v8_0_vm_decode_fault(adev, status, addr, mc_client);
+	gmc_v7_0_vm_decode_fault(adev, status, addr, mc_client);
 
 	return 0;
 }
 
-static int gmc_v8_0_set_clockgating_state(void *handle,
+static int gmc_v7_0_set_clockgating_state(void *handle,
 					  enum amd_clockgating_state state)
 {
+	bool gate = false;
+	struct amdgpu_device *adev = (struct amdgpu_device *)handle;
+
+	if (state == AMD_CG_STATE_GATE)
+		gate = true;
+
+	if (!(adev->flags & AMD_IS_APU)) {
+		gmc_v7_0_enable_mc_mgcg(adev, gate);
+		gmc_v7_0_enable_mc_ls(adev, gate);
+	}
+	gmc_v7_0_enable_bif_mgls(adev, gate);
+	gmc_v7_0_enable_hdp_mgcg(adev, gate);
+	gmc_v7_0_enable_hdp_ls(adev, gate);
+
 	return 0;
 }
 
-static int gmc_v8_0_set_powergating_state(void *handle,
+static int gmc_v7_0_set_powergating_state(void *handle,
 					  enum amd_powergating_state state)
 {
 	return 0;
 }
 
-const struct amd_ip_funcs gmc_v8_0_ip_funcs = {
-	.early_init = gmc_v8_0_early_init,
-	.late_init = gmc_v8_0_late_init,
-	.sw_init = gmc_v8_0_sw_init,
-	.sw_fini = gmc_v8_0_sw_fini,
-	.hw_init = gmc_v8_0_hw_init,
-	.hw_fini = gmc_v8_0_hw_fini,
-	.suspend = gmc_v8_0_suspend,
-	.resume = gmc_v8_0_resume,
-	.is_idle = gmc_v8_0_is_idle,
-	.wait_for_idle = gmc_v8_0_wait_for_idle,
-	.soft_reset = gmc_v8_0_soft_reset,
-	.print_status = gmc_v8_0_print_status,
-	.set_clockgating_state = gmc_v8_0_set_clockgating_state,
-	.set_powergating_state = gmc_v8_0_set_powergating_state,
+const struct amd_ip_funcs gmc_v7_0_ip_funcs = {
+	.early_init = gmc_v7_0_early_init,
+	.late_init = gmc_v7_0_late_init,
+	.sw_init = gmc_v7_0_sw_init,
+	.sw_fini = gmc_v7_0_sw_fini,
+	.hw_init = gmc_v7_0_hw_init,
+	.hw_fini = gmc_v7_0_hw_fini,
+	.suspend = gmc_v7_0_suspend,
+	.resume = gmc_v7_0_resume,
+	.is_idle = gmc_v7_0_is_idle,
+	.wait_for_idle = gmc_v7_0_wait_for_idle,
+	.soft_reset = gmc_v7_0_soft_reset,
+	.print_status = gmc_v7_0_print_status,
+	.set_clockgating_state = gmc_v7_0_set_clockgating_state,
+	.set_powergating_state = gmc_v7_0_set_powergating_state,
 };
 
-static const struct amdgpu_gart_funcs gmc_v8_0_gart_funcs = {
-	.flush_gpu_tlb = gmc_v8_0_gart_flush_gpu_tlb,
-	.set_pte_pde = gmc_v8_0_gart_set_pte_pde,
+static const struct amdgpu_gart_funcs gmc_v7_0_gart_funcs = {
+	.flush_gpu_tlb = gmc_v7_0_gart_flush_gpu_tlb,
+	.set_pte_pde = gmc_v7_0_gart_set_pte_pde,
 };
 
-static const struct amdgpu_irq_src_funcs gmc_v8_0_irq_funcs = {
-	.set = gmc_v8_0_vm_fault_interrupt_state,
-	.process = gmc_v8_0_process_interrupt,
+static const struct amdgpu_irq_src_funcs gmc_v7_0_irq_funcs = {
+	.set = gmc_v7_0_vm_fault_interrupt_state,
+	.process = gmc_v7_0_process_interrupt,
 };
 
-static void gmc_v8_0_set_gart_funcs(struct amdgpu_device *adev)
+static void gmc_v7_0_set_gart_funcs(struct amdgpu_device *adev)
 {
 	if (adev->gart.gart_funcs == NULL)
-		adev->gart.gart_funcs = &gmc_v8_0_gart_funcs;
+		adev->gart.gart_funcs = &gmc_v7_0_gart_funcs;
 }
 
-static void gmc_v8_0_set_irq_funcs(struct amdgpu_device *adev)
+static void gmc_v7_0_set_irq_funcs(struct amdgpu_device *adev)
 {
 	adev->mc.vm_fault.num_types = 1;
-	adev->mc.vm_fault.funcs = &gmc_v8_0_irq_funcs;
+	adev->mc.vm_fault.funcs = &gmc_v7_0_irq_funcs;
 }
